
/*****************************************************************
 *
 * SHA-1 sobre SIMD - Versión C Escalar.
 *
 * Pino, Jorge Daniel - JDanielPino@gmail.com
 *
 ****************************************************************/

#include <stdio.h>	/* size_t */
#include "sha1_c.h"

//
// Devuelve el hash SHA-1 de 160 bits.
//
// Parámetros:
//
// 1) uchar* buffer: puntero al buffer con los datos ya
//    rellenados con el 'padding'.
// 2) size_t size: tamaño en bytes del buffer.
// 3) uint hash[5]: vector donde la función devuelve el hash.
//
void get_sha1_c(uchar* buffer, size_t size, uint h[5]) {
  uint num_blocks = size / 64;
  uint w[80], i = 0, j = 0, a, b, c, d, e;

  h[0] = 0x67452301;
  h[1] = 0xEFCDAB89;
  h[2] = 0x98BADCFE;
  h[3] = 0x10325476;
  h[4] = 0xC3D2E1F0;

  for (i = 0; i < num_blocks; i++) {
    a = h[0];
    b = h[1];
    c = h[2];
    d = h[3];
    e = h[4];

    //
    // Para 0 <= j <= 19
    #define CALCULATE_W CALCULATE_W_00_TO_15
    #define F F1
    #define K K1

    UPDATE_ABCDE(0,  w, (buffer + i*64), a, b, c, d, e);
    UPDATE_ABCDE(1,  w, (buffer + i*64), e, a, b, c, d);
    UPDATE_ABCDE(2,  w, (buffer + i*64), d, e, a, b, c);
    UPDATE_ABCDE(3,  w, (buffer + i*64), c, d, e, a, b);
    UPDATE_ABCDE(4,  w, (buffer + i*64), b, c, d, e, a);
    UPDATE_ABCDE(5,  w, (buffer + i*64), a, b, c, d, e);
    UPDATE_ABCDE(6,  w, (buffer + i*64), e, a, b, c, d);
    UPDATE_ABCDE(7,  w, (buffer + i*64), d, e, a, b, c);
    UPDATE_ABCDE(8,  w, (buffer + i*64), c, d, e, a, b);
    UPDATE_ABCDE(9,  w, (buffer + i*64), b, c, d, e, a);
    UPDATE_ABCDE(10, w, (buffer + i*64), a, b, c, d, e);
    UPDATE_ABCDE(11, w, (buffer + i*64), e, a, b, c, d);
    UPDATE_ABCDE(12, w, (buffer + i*64), d, e, a, b, c);
    UPDATE_ABCDE(13, w, (buffer + i*64), c, d, e, a, b);
    UPDATE_ABCDE(14, w, (buffer + i*64), b, c, d, e, a);
    UPDATE_ABCDE(15, w, (buffer + i*64), a, b, c, d, e);

    #undef  CALCULATE_W
    #define CALCULATE_W CALCULATE_W_16_TO_79

    UPDATE_ABCDE(16, w, (buffer + i*64), e, a, b, c, d);
    UPDATE_ABCDE(17, w, (buffer + i*64), d, e, a, b, c);
    UPDATE_ABCDE(18, w, (buffer + i*64), c, d, e, a, b);
    UPDATE_ABCDE(19, w, (buffer + i*64), b, c, d, e, a);

    //
    // Para 20 <= j <= 39
    #undef  F
    #define F F2
    #undef  K
    #define K K2

    UPDATE_ABCDE(20, w, (buffer + i*64), a, b, c, d, e);
    UPDATE_ABCDE(21, w, (buffer + i*64), e, a, b, c, d);
    UPDATE_ABCDE(22, w, (buffer + i*64), d, e, a, b, c);
    UPDATE_ABCDE(23, w, (buffer + i*64), c, d, e, a, b);
    UPDATE_ABCDE(24, w, (buffer + i*64), b, c, d, e, a);
    UPDATE_ABCDE(25, w, (buffer + i*64), a, b, c, d, e);
    UPDATE_ABCDE(26, w, (buffer + i*64), e, a, b, c, d);
    UPDATE_ABCDE(27, w, (buffer + i*64), d, e, a, b, c);
    UPDATE_ABCDE(28, w, (buffer + i*64), c, d, e, a, b);
    UPDATE_ABCDE(29, w, (buffer + i*64), b, c, d, e, a);
    UPDATE_ABCDE(30, w, (buffer + i*64), a, b, c, d, e);
    UPDATE_ABCDE(31, w, (buffer + i*64), e, a, b, c, d);
    UPDATE_ABCDE(32, w, (buffer + i*64), d, e, a, b, c);
    UPDATE_ABCDE(33, w, (buffer + i*64), c, d, e, a, b);
    UPDATE_ABCDE(34, w, (buffer + i*64), b, c, d, e, a);
    UPDATE_ABCDE(35, w, (buffer + i*64), a, b, c, d, e);
    UPDATE_ABCDE(36, w, (buffer + i*64), e, a, b, c, d);
    UPDATE_ABCDE(37, w, (buffer + i*64), d, e, a, b, c);
    UPDATE_ABCDE(38, w, (buffer + i*64), c, d, e, a, b);
    UPDATE_ABCDE(39, w, (buffer + i*64), b, c, d, e, a);

    //
    // Para 40 <= j <= 59
    #undef  F
    #define F F3
    #undef  K
    #define K K3

    UPDATE_ABCDE(40, w, (buffer + i*64), a, b, c, d, e);
    UPDATE_ABCDE(41, w, (buffer + i*64), e, a, b, c, d);
    UPDATE_ABCDE(42, w, (buffer + i*64), d, e, a, b, c);
    UPDATE_ABCDE(43, w, (buffer + i*64), c, d, e, a, b);
    UPDATE_ABCDE(44, w, (buffer + i*64), b, c, d, e, a);
    UPDATE_ABCDE(45, w, (buffer + i*64), a, b, c, d, e);
    UPDATE_ABCDE(46, w, (buffer + i*64), e, a, b, c, d);
    UPDATE_ABCDE(47, w, (buffer + i*64), d, e, a, b, c);
    UPDATE_ABCDE(48, w, (buffer + i*64), c, d, e, a, b);
    UPDATE_ABCDE(49, w, (buffer + i*64), b, c, d, e, a);
    UPDATE_ABCDE(50, w, (buffer + i*64), a, b, c, d, e);
    UPDATE_ABCDE(51, w, (buffer + i*64), e, a, b, c, d);
    UPDATE_ABCDE(52, w, (buffer + i*64), d, e, a, b, c);
    UPDATE_ABCDE(53, w, (buffer + i*64), c, d, e, a, b);
    UPDATE_ABCDE(54, w, (buffer + i*64), b, c, d, e, a);
    UPDATE_ABCDE(55, w, (buffer + i*64), a, b, c, d, e);
    UPDATE_ABCDE(56, w, (buffer + i*64), e, a, b, c, d);
    UPDATE_ABCDE(57, w, (buffer + i*64), d, e, a, b, c);
    UPDATE_ABCDE(58, w, (buffer + i*64), c, d, e, a, b);
    UPDATE_ABCDE(59, w, (buffer + i*64), b, c, d, e, a);

    //
    // Para 60 <= j <= 79
    #undef  F
    #define F F4
    #undef  K
    #define K K4

    UPDATE_ABCDE(60, w, (buffer + i*64), a, b, c, d, e);
    UPDATE_ABCDE(61, w, (buffer + i*64), e, a, b, c, d);
    UPDATE_ABCDE(62, w, (buffer + i*64), d, e, a, b, c);
    UPDATE_ABCDE(63, w, (buffer + i*64), c, d, e, a, b);
    UPDATE_ABCDE(64, w, (buffer + i*64), b, c, d, e, a);
    UPDATE_ABCDE(65, w, (buffer + i*64), a, b, c, d, e);
    UPDATE_ABCDE(66, w, (buffer + i*64), e, a, b, c, d);
    UPDATE_ABCDE(67, w, (buffer + i*64), d, e, a, b, c);
    UPDATE_ABCDE(68, w, (buffer + i*64), c, d, e, a, b);
    UPDATE_ABCDE(69, w, (buffer + i*64), b, c, d, e, a);
    UPDATE_ABCDE(70, w, (buffer + i*64), a, b, c, d, e);
    UPDATE_ABCDE(71, w, (buffer + i*64), e, a, b, c, d);
    UPDATE_ABCDE(72, w, (buffer + i*64), d, e, a, b, c);
    UPDATE_ABCDE(73, w, (buffer + i*64), c, d, e, a, b);
    UPDATE_ABCDE(74, w, (buffer + i*64), b, c, d, e, a);
    UPDATE_ABCDE(75, w, (buffer + i*64), a, b, c, d, e);
    UPDATE_ABCDE(76, w, (buffer + i*64), e, a, b, c, d);
    UPDATE_ABCDE(77, w, (buffer + i*64), d, e, a, b, c);
    UPDATE_ABCDE(78, w, (buffer + i*64), c, d, e, a, b);
    UPDATE_ABCDE(79, w, (buffer + i*64), b, c, d, e, a);

    h[0] += a;
    h[1] += b;
    h[2] += c;
    h[3] += d;
    h[4] += e;
  }
}
