#include <stdio.h>  /* size_t */
#include "sha1_c_normal_loop.h"

//
// Devuelve el hash SHA-1 de 160 bits.
//
// Parámetros:
//
// 1) uchar* buffer: puntero al buffer con los datos ya
//    rellenados con el 'padding'.
// 2) size_t size: tamaño en bytes del buffer.
// 3) uint hash[5]: vector donde la función devuelve el hash.
//
void get_sha1_c_normal_loop(uchar* buffer, size_t size, uint h[5]) {
  uint num_blocks = size / 64;
  uint w[80], i = 0, j = 0, a, b, c, d, e, f, k;

  h[0] = 0x67452301;
  h[1] = 0xEFCDAB89;
  h[2] = 0x98BADCFE;
  h[3] = 0x10325476;
  h[4] = 0xC3D2E1F0;

  for (i = 0; i < num_blocks; i++) {
    a = h[0];
    b = h[1];
    c = h[2];
    d = h[3];
    e = h[4];

    // Ciclo principal
    for (j = 0; j < 80; j++) {

      if (j < 16) {
	w[j] = (buffer[i*64 + 4*j]     << 24) |
 	       (buffer[i*64 + 4*j + 1] << 16) |
 	       (buffer[i*64 + 4*j + 2] << 8)  |
 	       (buffer[i*64 + 4*j + 3]);
      } else {
	w[j] = ROTATE_LEFT((w[j-3] ^ w[j-8] ^ w[j-14] ^ w[j-16]), 1);
      }

      if (j <= 19) {
	f = (b & c) | ((~b) & d);
	k = 0x5A827999;
      }
      else if (j <= 39) {
	f = b ^ c ^ d;
	k = 0x6ED9EBA1;
      }
      else if (j <= 59) {
	f = (b & c) | (b & d) | (c & d);
	k = 0x8F1BBCDC;
      }
      else if (60 <= j) {
	f = (b ^ c) ^ d;
	k = 0xCA62C1D6;
      }

      uint temp = ROTATE_LEFT(a, 5) + f + e + k + w[j];
      e = d;
      d = c;
      c = ROTATE_LEFT(b, 30);
      b = a;
      a = temp;
    }

    h[0] = h[0] + a;
    h[1] = h[1] + b;
    h[2] = h[2] + c;
    h[3] = h[3] + d;
    h[4] = h[4] + e;
  }
}
