//
// Alias para los tipos
//
typedef unsigned int  uint;
typedef unsigned long ulong;
typedef unsigned char uchar;

//
// Macros
//
#define ROTATE_LEFT(v, n) \
({\
  unsigned int _aux1, _aux2, _n; \
  _n = n % (sizeof(unsigned int)*8); \
  _aux1 = v << _n; \
  _aux2 = v >> ((sizeof(unsigned int)*8) - _n); \
  (_aux1 | _aux2); \
})

//
// Macros asociadas a las funciones y constantes para la
// actualización de las variables A, B, C, D y E.
//
#define F1(B, C, D) ((B & C) | ((~B) & D))
#define F2(B, C, D) (B ^ C ^ D)
#define F3(B, C, D) ((B & C) | (B & D) | (C & D))
#define F4 F2

#define K1 0x5A827999
#define K2 0x6ED9EBA1
#define K3 0x8F1BBCDC
#define K4 0xCA62C1D6

//
// Calcula el elemento h del vector w_ptr para el caso en que
// 0 <= h < 16.
#define CALCULATE_W_00_TO_15(h, w_ptr, buffer_ptr)\
({\
  (buffer_ptr[4*h]     << 24) | \
  (buffer_ptr[4*h + 1] << 16) | \
  (buffer_ptr[4*h + 2] << 8)  | \
  (buffer_ptr[4*h + 3]); \
})

//
// Calcula el elemento h del vector w_ptr para el caso en que
// 15 < h < 79.
#define CALCULATE_W_16_TO_79(h, w_ptr, buffer_ptr)\
({\
  (ROTATE_LEFT((w_ptr[h-3] ^ w_ptr[h-8] ^ w_ptr[h-14] ^ w_ptr[h-16]), 1)); \
})

//
// Actualiza las variables A, B, C, D y E que van determinando el
// valor de hash en cada ronda. Requiere que se defina
// previamente la macro-función 'CALCULATE_W' (que debe ser
// CALCULATE_W_00_TO_15 para valores de h entre 0 y 16, y
// CALCULATE_W_16_TO_79, para h entre 16 y 79), la función 'F'
// (que debe ser F1 para h entre 0 y 19, F2 para h entre 20 y 39,
// F3 para h entre 40 y 59, y F4 para h entre 60 y 79), y la
// constante 'K' (que debe ser K1 para h entre 0 y 19, K2 para h
// entre 20 y 39, K3 para h entre 40 y 59, y K4 para h entre 60 y
// 79.
#define UPDATE_ABCDE(h, w_ptr, buffer_ptr, A, B, C, D, E) \
({\
  w_ptr[h] = CALCULATE_W(h, w_ptr, buffer_ptr); \
  E += ROTATE_LEFT(A, 5) + F(B, C, D) + w_ptr[h] + K; \
  B =  ROTATE_LEFT(B, 30); \
})


//
// Devuelve el hash SHA-1 de 160 bits.
//
// Parámetros:
//
// 1) uchar* buffer: puntero al buffer con los datos ya
//    rellenados con el 'padding'.
// 2) size_t size: tamaño en bytes del buffer.
// 3) uint hash[5]: vector donde la función devuelve el hash.
//
void get_sha1_c_unrolled_loop(uchar* buffer, size_t size, uint h[5]);

